/**
 * Copyright (c) 2002-2014 "Neo Technology,"
 * Network Engine for Objects in Lund AB [http://neotechnology.com]
 *
 * This file is part of Neo4j.
 *
 * Neo4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.neo4j.cypher.internal.compiler.v2_1.commands

import expressions.Expression
import org.neo4j.cypher.internal.compiler.v2_1._
import pipes.QueryState
import org.neo4j.cypher.PatternException

case class SortItem(expression: Expression, ascending: Boolean) {
  def apply(ctx: ExecutionContext)(implicit qtx: QueryState) =
    if (!expression.isDeterministic)
      throw new PatternException("ORDER BY expressions must be deterministic. " +
        "For instance, you cannot use the rand() function in the expression")
    else
      expression.apply(ctx)
}
